import {
	Expose, Exclude
} from 'class-transformer';

export default class BingoGroup {
	name: string;

	#parent?: BingoGroup;

	constructor(name: string, parent?: BingoGroup) {
		this.name = name;
		this.parent = parent;
	}

	@Exclude()
	get parent(): BingoGroup | undefined {
		return this.#parent;
	}

	set parent(newParent: BingoGroup | undefined) {
		if (newParent === this) {
			throw new Error('A BingoGroup cannot be a parent of itself.');
		}

		// if (!newParent) {
		// 	this.#parent = undefined;

		// 	return;
		// }

		let temp = newParent;
		while (temp) {
			if (temp === this) {
				throw new Error('Circular reference detected.');
			}
			temp = temp.parent;
		}

		this.#parent = newParent;
	}

	@Expose()
	get parent_id(): string | undefined {
		return this.#parent?.name;
	}
}

